@@ -1207,8 +1207,10 @@ class SaleLine(ModelSQL, ModelView):
         invoice_line.type = self.type
         invoice_line.description = self.description
         invoice_line.note = self.note
+        invoice_line.origin = self
         if self.type != 'line':
             if (self.sale.invoice_method == 'order'
+                    and not self.invoice_lines
                     and ((all(l.quantity >= 0 for l in self.sale.lines
                                 if l.type == 'line')
                             and invoice_type == 'out_invoice')
@@ -1241,10 +1243,12 @@ class SaleLine(ModelSQL, ModelView):
             if old_invoice_line.id not in skip_ids:
                 quantity -= Uom.compute_qty(old_invoice_line.unit,
                     old_invoice_line.quantity, self.unit)
-        invoice_line.quantity = quantity
 
-        if invoice_line.quantity <= 0.0:
+        rounding = self.unit.rounding if self.unit else 0.01
+        invoice_line.quantity = Uom.round(quantity, rounding)
+        if invoice_line.quantity <= 0:
             return []
+
         invoice_line.unit = self.unit
         invoice_line.product = self.product
         invoice_line.unit_price = self.unit_price
@@ -1265,7 +1269,6 @@ class SaleLine(ModelSQL, ModelView):
                 self.raise_user_error('missing_account_revenue_property', {
                         'sale': self.sale.rec_name,
                         })
-        invoice_line.origin = self
         return [invoice_line]
 
     @classmethod
@@ -1310,8 +1313,11 @@ class SaleLine(ModelSQL, ModelView):
             if move.id not in skip_ids:
                 quantity -= Uom.compute_qty(move.uom, move.quantity,
                     self.unit)
-        if quantity <= 0.0:
+
+        quantity = Uom.round(quantity, self.unit.rounding)
+        if quantity <= 0:
             return
+
         if not self.sale.party.customer_location:
             self.raise_user_error('customer_location_required', {
                     'sale': self.sale.rec_name,
